@@ -17,9 +17,10 @@
 import re
 import sys
 import importlib
+from calculate.lib.utils.tools import drop_slash
 from utils.text import convertStrListDict, _u, _u8, formatListOr
 from utils.files import pathJoin
-from utils.portage import searchProfile, RepositorySubstituting
+from utils.portage import searchProfile, RepositorySubstituting, RepositoryPath
 from os import path
 import os
 from collections import OrderedDict, Iterable
@@ -28,19 +29,21 @@ import pyinotify
 from itertools import *
 import operator
 import traceback
+from string import Formatter
 from contextlib import contextmanager
 from types import StringTypes, GeneratorType
 from calculate.lib.configparser import ConfigParser, ParsingError
 
-from calculate.lib.cl_lang import setLocalTranslate
+from calculate.lib.cl_lang import setLocalTranslate, getLazyLocalTranslate
 
 _ = lambda x: x
 setLocalTranslate('cl_lib3', sys.modules[__name__])
+__ = getLazyLocalTranslate(_)
 
 
 def addStdConfig(fileLst, prefix='/'):
     stdPath = path.join(prefix,
-                        "var/lib/layman/calculate/profiles/calculate.env")
+                        drop_slash(RepositoryPath.Calculate), "calculate.env")
     if stdPath not in map(os.path.abspath, fileLst):
         return [stdPath] + fileLst
     return fileLst
@@ -81,9 +84,6 @@ class CommonVariableError(VariableError):
     """Exception for command line"""
 
 
-import cl_ini_parser
-
-
 def makePath(dirs, mode=755):
     if not path.exists(dirs):
         os.makedirs(dirs, mode)
@@ -99,7 +99,20 @@ class HumanReadable(object):
     Auto = None
 
 
+class ValueFormatter(Formatter):
+    def get_field(self, field_name, args, kwargs):
+        Get = args[0]
+        if field_name.endswith("()"):
+            field_name, o, func = field_name.rpartition(".")
+            value = str(Get(field_name))
+            return getattr(value, func[:-2])(), None
+        return Get(field_name), None
+
+
 class VariableInterface(object):
+    _value_formatter = ValueFormatter()
+    type = "string"
+
     def Get(self, varname=None, humanreadable=False):
         pass
 
@@ -158,6 +171,7 @@ class Variable(VariableInterface):
         List = "list"
         Choice = "choice"
         Table = "table"
+        Boolean = "bool"
 
     # label of variable
     label = None
@@ -194,6 +208,11 @@ class Variable(VariableInterface):
     # при установке значения переменной устанавливает значение
     # указанной переменной
     alias = ""
+    # значение заполняется переменными
+    value_format = ""
+
+    # спец тип для gui
+    guitype = ""
 
     On = "on"
     Off = "off"
@@ -241,6 +260,14 @@ class Variable(VariableInterface):
         else:
             return "s"
 
+    @property
+    def empty_value(self):
+        if "list" in self.type:
+            return []
+        elif "table" in self.type:
+            return [[]]
+        return ""
+
     def GetBool(self, varname):
         return self.isTrue(self.Get(varname))
 
@@ -292,11 +319,17 @@ class Variable(VariableInterface):
         """
         return zip(*map(self.Get, argvVarNames))
 
+    def _get_format_value(self):
+        return self._value_formatter.format(
+            self.__class__.value_format, self.Get)
+
     def get(self):
         """
         Overridable
         Calculate value method
         """
+        if self.__class__.value_format:
+            return self._get_format_value()
         if self.__class__.value:
             return self.__class__.value
         else:
@@ -342,12 +375,13 @@ class Variable(VariableInterface):
             return False
         self.bIniRead = True
         try:
-            value = self.parent.iniCache[self.name]['value']
+            value = self.parent.iniCache[self.fullname]
             value = self.parent.unserialize(self.type,
                                             value.encode('utf-8'))
             if self.mode == READONLY:
                 raise VariableError(
-                    _("Attempt to rewrite readonly variable %s") %
+                    _("Attempting to rewrite readonly variable %s "
+                      "via calculate.env") %
                     self.name)
             else:
                 try:
@@ -358,6 +392,10 @@ class Variable(VariableInterface):
         finally:
             self.bIniRead = False
 
+    @property
+    def fullname(self):
+        return '%s.%s' % (self.section, self.name)
+
     def _get(self):
         """
         Standard inner method for getting value of variable.
@@ -365,7 +403,7 @@ class Variable(VariableInterface):
         # variable need refill
         if self.invalid:
             # get net value
-            if self.name in self.parent.iniCache and self.iniread():
+            if self.fullname in self.parent.iniCache and self.iniread():
                 return self.value
             else:
                 oldModeGet = self.modeGet
@@ -406,12 +444,23 @@ class Variable(VariableInterface):
         else:
             return res, None
 
+    def check_on(self):
+        pass
+
+    def check_off(self):
+        pass
+
     def check(self, value):
         """
         Overridable
 
         Check method, for wrong value need raise VariableError
         """
+        if self.type == self.Types.Boolean:
+            if self.isTrue(value):
+                self.check_on()
+            else:
+                self.check_off()
         pass
 
     def uncompatible(self):
@@ -464,7 +513,8 @@ class Variable(VariableInterface):
     def checkType(self, value):
         """Check value for type"""
         if "int" in self.type:
-            if value and not value.isdigit():
+            if (value and not value.isdigit() and not value.startswith("-") and
+                    not value[1:].isdigit()):
                 raise VariableError(
                     _("The value of variable '{varname}' must be integer"
                       ).format(varname=self.label or self.name))
@@ -488,7 +538,7 @@ class Variable(VariableInterface):
                     vartype="string"))
             return
         if "choice" in self.type:
-            choiceVal = self.choice()
+            choiceVal = self.choice() or []
             if isinstance(choiceVal, GeneratorType):
                 choiceVal = tuple(choiceVal)
 
@@ -512,9 +562,10 @@ class Variable(VariableInterface):
                                 "{vartype}")
                     self.raiseWrongChoice(name, tipChoice, val, error)
                 if "choice" in self.type and not "choiceedit" in self.type:
-                    if (choiceVal and (val or val == "") and
-                                val not in choiceVal or
-                                val is None):
+                    if ((choiceVal and (val or val == "") and
+                                 val not in choiceVal or
+                                 val is None) or
+                            not choiceVal):
                         if not choiceVal:
                             self.raiseNothingValue()
                         self.raiseWrongChoice(name, tipChoice, val, error)
@@ -560,15 +611,16 @@ class Variable(VariableInterface):
         """
         Full check value
         """
-        if value is None:
-            value = self.Get(self.name)
-        try:
-            self.modeGet = Variable.CHECK
-            self.untrusted = True
-            self.check(value)
-            self.untrusted = False
-        finally:
-            self.modeGet = Variable.NONE
+        if self.name != "cl_help_set" and not self.GetBool('cl_help_set'):
+            if value is None:
+                value = self.Get(self.name)
+            try:
+                self.modeGet = Variable.CHECK
+                self.untrusted = True
+                self.check(value)
+                self.untrusted = False
+            finally:
+                self.modeGet = Variable.NONE
 
     def invalidate(self, force=False):
         """
@@ -599,8 +651,11 @@ class Variable(VariableInterface):
     def isTrue(cls, value):
         if type(value) == bool:
             return value
-        if value.lower() in ('yes', 'on', 'true'):
+        value = value.lower()
+        if value in ('yes', 'on', 'true'):
             return True
+        if value == 'auto':
+            return None
         return False
 
     def has_base_method(self, methodname):
@@ -713,7 +768,7 @@ class TableVariable(Variable):
         readonly index
         """
         raise DataVarsError(
-            _("Attempt to rewrite a readonly index field "
+            _("Attempting to rewrite a readonly index field "
               "{fieldname} in variable {variablename}").format(
                 fieldname=fieldname, variablename=variablename))
 
@@ -1366,10 +1421,10 @@ class DataVars(SimpleDataVars):
                     for section in iniSections:
                         for key, value in config.items(section, raw=True):
                             key = _u8(key)
-                            self.iniCache[key] = {'value': value,
-                                                  'section': section}
+                            fullkey = "%s.%s" % (section, key)
+                            self.iniCache[fullkey] = value
                             if iniName != "profile":
-                                self.filledVars[key] = iniName
+                                self.filledVars[fullkey] = iniName
         return True
 
     def Get(self, varname, humanreadable=HumanReadable.No):
@@ -1386,6 +1441,7 @@ class DataVars(SimpleDataVars):
         Check what value get from ini file
         """
         section, varname = self.splitVarname(varname)
+        varname = "%s.%s" % (section, varname)
         return self.filledVars[varname] if varname in self.filledVars else ""
 
     def splitVarname(self, varname):
@@ -1525,7 +1581,7 @@ class DataVars(SimpleDataVars):
                 self.Set(varObj.alias, varObj.value, force)
         else:
             raise DataVarsError(
-                _("Attempt to rewrite readonly variable %s") %
+                _("Attempting to rewrite readonly variable %s") %
                 varname)
 
     def Invalidate(self, varname, force=False, onlySet=False):
@@ -1580,6 +1636,7 @@ class DataVars(SimpleDataVars):
             if not header:
                 header = self.loadVariables[varname].section
             # write value to ini file
+            import cl_ini_parser
             config = cl_ini_parser.iniParser(iniFile)
             val = self.serialize(self.getInfo(varname).type,
                                  val)
@@ -1614,6 +1671,7 @@ class DataVars(SimpleDataVars):
                     self.loadVariable(varname)
                 header = self.loadVariables[varname].section
             # write value to ini file
+            import cl_ini_parser
             config = cl_ini_parser.iniParser(iniFile)
             allVars = config.getAreaVars(header)
             if allVars is False:
@@ -1784,7 +1842,7 @@ class DataVars(SimpleDataVars):
             print "|", vname, "=", self.Get(vname)
         print "=DEBUG.END="
 
-    def checkGroups(self, info, allvars=False):
+    def checkGroups(self, info, allvars=False, invalidators=None):
         """
         Check variables in group or all in info
         """
@@ -1830,6 +1888,17 @@ class DataVars(SimpleDataVars):
                 self.Invalidate(varname, onlySet=True)
         else:
             default = []
+        # проверить переменные, при смене значения которых нужно сбросить
+        # указанные зависимые, даже если пользователь уже изменял их значения
+        # это может быть использово при groupmod
+        if invalidators:
+            for k, vars in invalidators.items():
+                if hasattr(info, k):
+                    val = getattr(info,k)
+                    if val is not None and val != self.Get(k):
+                        for var in vars:
+                            self.Invalidate(var, force=True)
+
         # look over all vars
         for var, varObj in map(lambda x: (x.name, x),
                                self._nocheckSort(self._dependSort(varsByKeys))):
@@ -1842,6 +1911,8 @@ class DataVars(SimpleDataVars):
                     continue
                 # get value of variable from info
                 val = getattr(info, var)
+                if type(val) == unicode:
+                    val = _u8(val)
             varSendFromClient = val is not None and var not in default
             varGroupUntrusted = varObj.untrusted and var in groupVars
             varAllUntrusted = varObj.untrusted and allvars
