@@ -770,7 +770,7 @@ class VariableClMakeProfile(Variable):
                 _("Failed to detect the system profile.") + " " +
                 _("Select the profile with command {cmd}").format(
                     cmd="cl-update-profile"))
-        return val
+        return val or ""
 
 
 class VariableClHelpSet(ReadonlyVariable):
